@@ -87,9 +87,23 @@
 #![warn(missing_docs)]
 #![deny(warnings)]
 
-#[cfg(unix)]
+#[cfg(any(
+    target_os = "linux",
+    target_os = "macos",
+    target_os = "dragonfly",
+    target_os = "freebsd",
+    target_os = "openbsd",
+    target_os = "netbsd"
+))]
 pub mod unix;
-#[cfg(unix)]
+#[cfg(any(
+    target_os = "linux",
+    target_os = "macos",
+    target_os = "dragonfly",
+    target_os = "freebsd",
+    target_os = "openbsd",
+    target_os = "netbsd"
+))]
 pub use unix::*;
 
 #[cfg(windows)]
@@ -98,7 +112,7 @@ pub mod windows;
 pub use windows::*;
 
 /// A error type
-#[derive(Debug, Copy, Clone)]
+#[derive(Debug, Copy, Clone, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub enum Error {
     /// A value which describes why it is impossible to use such a priority.
     Priority(&'static str),
@@ -112,8 +126,35 @@ pub enum Error {
 /// Platform-independent thread priority value.
 /// Should be in `[0; 100)` range. The higher the number is - the higher
 /// the priority.
+///
+/// The only way to create such a value is a safe conversion from an 8-byte
+/// unsigned integer ([`u8`]):
+///
+/// ```rust
+/// use thread_priority::*;
+/// use std::convert::{TryFrom, TryInto};
+///
+/// // Create the lowest possible priority value.
+/// assert!(ThreadPriorityValue::try_from(0u8).is_ok());
+/// // Create it implicitly via `TryInto`:
+/// let _priority = ThreadPriority::Crossplatform(0u8.try_into().unwrap());
+/// ```
+///
+/// In case you need to get the raw value out of it, use the `Into<u8>` trait:
+///
+/// ```rust
+/// use thread_priority::*;
+/// use std::convert::TryFrom;
+///
+/// // Create the lowest possible priority value.
+/// let priority = ThreadPriorityValue::try_from(0u8).unwrap();
+/// // Create it implicitly via `TryInto`:
+/// let raw_value: u8 = priority.into();
+/// assert_eq!(raw_value, 0);
+/// ```
 #[derive(Copy, Clone, Debug, Default, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub struct ThreadPriorityValue(u8);
+
 impl std::convert::TryFrom<u8> for ThreadPriorityValue {
     type Error = &'static str;
 
@@ -126,6 +167,15 @@ impl std::convert::TryFrom<u8> for ThreadPriorityValue {
     }
 }
 
+// The From<u8> is unsafe, so there is a TryFrom instead.
+// For this reason we silent the warning from clippy.
+#[allow(clippy::from_over_into)]
+impl std::convert::Into<u8> for ThreadPriorityValue {
+    fn into(self) -> u8 {
+        self.0
+    }
+}
+
 /// Platform-specific thread priority value.
 #[derive(Copy, Clone, Debug, Default, Eq, PartialEq, Ord, PartialOrd, Hash)]
 pub struct ThreadPriorityOsValue(u32);
@@ -134,6 +184,14 @@ pub struct ThreadPriorityOsValue(u32);
 #[derive(Debug, Copy, Clone, Hash, Eq, PartialEq, Ord, PartialOrd)]
 pub enum ThreadPriority {
     /// Holds a value representing the minimum possible priority.
+    #[cfg_attr(
+        target_os = "windows",
+        doc = "\
+The [`ThreadPriority::Min`] value is mapped to [`WinAPIThreadPriority::Lowest`] and not
+[`WinAPIThreadPriority::Idle`] to avoid unexpected drawbacks. Use the specific value
+to set it to [`WinAPIThreadPriority::Idle`] when it is really needed.
+"
+    )]
     Min,
     /// Holds a platform-independent priority value.
     /// Usually used when setting a value, for sometimes it is not possible to map
@@ -141,6 +199,15 @@ pub enum ThreadPriority {
     Crossplatform(ThreadPriorityValue),
     /// Holds an operating system specific value. If it is not possible to obtain the
     /// [`ThreadPriority::Crossplatform`] variant of the value, this is returned instead.
+    #[cfg_attr(
+        target_os = "windows",
+        doc = "\
+The value is matched among possible values in Windows from [`WinAPIThreadPriority::Idle`] till
+[`WinAPIThreadPriority::TimeCritical`]. This is due to windows only having from 7 to 9 possible
+thread priorities and not `100` as it is allowed to have in the [`ThreadPriority::Crossplatform`]
+variant.
+"
+    )]
     Os(ThreadPriorityOsValue),
     /// Holds scheduling parameters for Deadline scheduling. These are, in order,
     /// the nanoseconds for runtime, deadline, and period. Please note that the
@@ -152,6 +219,14 @@ pub enum ThreadPriority {
     /// os where the program is going to be running on, how it will
     /// behave. On some systems, the whole system may become frozen
     /// if not used properly.
+    #[cfg_attr(
+        target_os = "windows",
+        doc = "\
+The [`ThreadPriority::Max`] value is mapped to [`WinAPIThreadPriority::Highest`] and not
+[`WinAPIThreadPriority::TimeCritical`] to avoid unexpected drawbacks. Use the specific value
+to set it to [`WinAPIThreadPriority::TimeCritical`] when it is really needed.
+"
+    )]
     Max,
 }
 
@@ -183,7 +258,7 @@ impl Thread {
     /// ```
     pub fn current() -> Result<Thread, Error> {
         Ok(Thread {
-            priority: thread_priority()?,
+            priority: get_current_thread_priority()?,
             id: thread_native_id(),
         })
     }
