@@ -6,12 +6,32 @@
 
 use std::convert::TryFrom;
 
+use const_format::concatcp;
+
 use crate::{Error, ThreadPriority, ThreadPriorityValue};
+use std::mem::MaybeUninit;
+
+#[cfg(not(target_os = "macos"))]
+const SCHED_FIFO: i32 = 1;
+#[cfg(target_os = "macos")]
+const SCHED_FIFO: i32 = 4;
+
+// Processes scheduled under one of the real-time policies
+// (SCHED_FIFO, SCHED_RR) have a sched_priority value in the range 1
+// (low) to 99 (high).
+const MIN_PRIORITY: i32 = 1;
+const MAX_PRIORITY: i32 = 99;
+// For threads scheduled under one of the normal scheduling policies
+//  (SCHED_OTHER, SCHED_IDLE, SCHED_BATCH), sched_priority is not
+//  used in scheduling decisions (it must be specified as 0).
+// <https://man7.org/linux/man-pages/man7/sched.7.html>
+const NORMAL_PRIORITY: i32 = 0;
 
 /// An alias type for a thread id.
 pub type ThreadId = libc::pthread_t;
 
 /// Proxy structure to maintain compatibility between glibc and musl
+#[derive(Debug, Copy, Clone, Eq, PartialEq, Ord, PartialOrd)]
 pub struct ScheduleParams {
     /// Copy of `sched_priority` from `libc::sched_param`
     pub sched_priority: libc::c_int,
@@ -45,30 +65,10 @@ pub struct SchedAttr {
 }
 
 impl ScheduleParams {
-    #[cfg(not(target_env = "musl"))]
     fn into_posix(self) -> libc::sched_param {
-        libc::sched_param {
-            sched_priority: self.sched_priority,
-        }
-    }
-
-    #[cfg(target_env = "musl")]
-    fn into_posix(self) -> libc::sched_param {
-        use libc::timespec as TimeSpec;
-
-        libc::sched_param {
-            sched_priority: self.sched_priority,
-            sched_ss_low_priority: 0,
-            sched_ss_repl_period: TimeSpec {
-                tv_sec: 0,
-                tv_nsec: 0,
-            },
-            sched_ss_init_budget: TimeSpec {
-                tv_sec: 0,
-                tv_nsec: 0,
-            },
-            sched_ss_max_repl: 0,
-        }
+        let mut param = unsafe { MaybeUninit::<libc::sched_param>::zeroed().assume_init() };
+        param.sched_priority = self.sched_priority;
+        param
     }
 
     fn from_posix(sched_param: libc::sched_param) -> Self {
@@ -92,10 +92,11 @@ pub enum RealtimeThreadSchedulePolicy {
     #[cfg(target_os = "linux")]
     Deadline,
 }
+
 impl RealtimeThreadSchedulePolicy {
     fn to_posix(self) -> libc::c_int {
         match self {
-            RealtimeThreadSchedulePolicy::Fifo => 1,
+            RealtimeThreadSchedulePolicy::Fifo => SCHED_FIFO,
             RealtimeThreadSchedulePolicy::RoundRobin => 2,
             #[cfg(target_os = "linux")]
             RealtimeThreadSchedulePolicy::Deadline => 6,
@@ -116,6 +117,7 @@ pub enum NormalThreadSchedulePolicy {
     Normal,
 }
 impl NormalThreadSchedulePolicy {
+    #[cfg(not(target_os = "macos"))]
     fn to_posix(self) -> libc::c_int {
         match self {
             NormalThreadSchedulePolicy::Idle => 5,
@@ -123,6 +125,14 @@ impl NormalThreadSchedulePolicy {
             NormalThreadSchedulePolicy::Other | NormalThreadSchedulePolicy::Normal => 0,
         }
     }
+
+    #[cfg(target_os = "macos")]
+    fn to_posix(self) -> libc::c_int {
+        match self {
+            NormalThreadSchedulePolicy::Other => 1,
+            _ => panic!("Invalid value for berkley schedule policy."),
+        }
+    }
 }
 
 /// Thread schedule policy definition
@@ -141,6 +151,7 @@ impl ThreadSchedulePolicy {
         }
     }
 
+    #[cfg(not(target_os = "macos"))]
     fn from_posix(policy: libc::c_int) -> Result<ThreadSchedulePolicy, Error> {
         match policy {
             0 => Ok(ThreadSchedulePolicy::Normal(
@@ -152,7 +163,7 @@ impl ThreadSchedulePolicy {
             5 => Ok(ThreadSchedulePolicy::Normal(
                 NormalThreadSchedulePolicy::Idle,
             )),
-            1 => Ok(ThreadSchedulePolicy::Realtime(
+            SCHED_FIFO => Ok(ThreadSchedulePolicy::Realtime(
                 RealtimeThreadSchedulePolicy::Fifo,
             )),
             2 => Ok(ThreadSchedulePolicy::Realtime(
@@ -165,6 +176,24 @@ impl ThreadSchedulePolicy {
             _ => Err(Error::Ffi("Can't parse schedule policy from posix")),
         }
     }
+
+    #[cfg(target_os = "macos")]
+    fn from_posix(policy: libc::c_int) -> Result<ThreadSchedulePolicy, Error> {
+        match policy {
+            1 => Ok(ThreadSchedulePolicy::Normal(
+                NormalThreadSchedulePolicy::Other,
+            )),
+            SCHED_FIFO => Ok(ThreadSchedulePolicy::Realtime(
+                RealtimeThreadSchedulePolicy::Fifo,
+            )),
+            2 => Ok(ThreadSchedulePolicy::Realtime(
+                RealtimeThreadSchedulePolicy::RoundRobin,
+            )),
+            _ => Err(Error::Ffi(
+                "Can't parse schedule policy from berkley values",
+            )),
+        }
+    }
 }
 
 impl ThreadPriority {
@@ -178,8 +207,8 @@ impl ThreadPriority {
                 ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => Err(
                     Error::Priority("Deadline scheduling must use deadline priority."),
                 ),
-                ThreadSchedulePolicy::Realtime(_) => Ok(1),
-                _ => Ok(0),
+                ThreadSchedulePolicy::Realtime(_) => Ok(MIN_PRIORITY as u32),
+                _ => Ok(NORMAL_PRIORITY as u32),
             },
             ThreadPriority::Crossplatform(ThreadPriorityValue(p)) => match policy {
                 // SCHED_DEADLINE doesn't really have a notion of priority, this is an error
@@ -187,12 +216,20 @@ impl ThreadPriority {
                 ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => Err(
                     Error::Priority("Deadline scheduling must use deadline priority."),
                 ),
-                ThreadSchedulePolicy::Realtime(_) if (p == 0 || p > 99) => {
-                    Err(Error::Priority("The value is out of range [0; 99]"))
+                ThreadSchedulePolicy::Realtime(_)
+                    if !((MIN_PRIORITY..=MAX_PRIORITY).contains(&(p as i32))) =>
+                {
+                    Err(Error::Priority(concatcp!(
+                        "The value is out of range [",
+                        MIN_PRIORITY,
+                        "; ",
+                        MAX_PRIORITY,
+                        "]"
+                    )))
                 }
-                ThreadSchedulePolicy::Normal(_) if p != 0 => Err(Error::Priority(
-                    "The value can be only 0 for normal scheduling policy",
-                )),
+                ThreadSchedulePolicy::Normal(_) if p as i32 != NORMAL_PRIORITY => Err(
+                    Error::Priority("The value can be only 0 for normal scheduling policy"),
+                ),
                 _ => Ok(p as u32),
             },
             // TODO avoid code duplication.
@@ -202,12 +239,24 @@ impl ThreadPriority {
                 ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => Err(
                     Error::Priority("Deadline scheduling must use deadline priority."),
                 ),
-                ThreadSchedulePolicy::Realtime(_) if (p == 0 || p > 99) => {
-                    Err(Error::Priority("The value is out of range [0; 99]"))
+                ThreadSchedulePolicy::Realtime(_)
+                    if !((MIN_PRIORITY..=MAX_PRIORITY).contains(&(p as i32))) =>
+                {
+                    Err(Error::Priority(concatcp!(
+                        "The value is out of range [",
+                        MIN_PRIORITY,
+                        "; ",
+                        MAX_PRIORITY,
+                        "]"
+                    )))
+                }
+                ThreadSchedulePolicy::Normal(_) if p as i32 != NORMAL_PRIORITY => {
+                    Err(Error::Priority(concatcp!(
+                        "The value can be only ",
+                        NORMAL_PRIORITY,
+                        " for normal scheduling policy"
+                    )))
                 }
-                ThreadSchedulePolicy::Normal(_) if p != 0 => Err(Error::Priority(
-                    "The value can be only 0 for normal scheduling policy",
-                )),
                 _ => Ok(p),
             },
             ThreadPriority::Max => match policy {
@@ -216,8 +265,8 @@ impl ThreadPriority {
                 ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => Err(
                     Error::Priority("Deadline scheduling must use deadline priority."),
                 ),
-                ThreadSchedulePolicy::Realtime(_) => Ok(99),
-                _ => Ok(0),
+                ThreadSchedulePolicy::Realtime(_) => Ok(MAX_PRIORITY as u32),
+                _ => Ok(NORMAL_PRIORITY as u32),
             },
             #[cfg(target_os = "linux")]
             ThreadPriority::Deadline(_, _, _) => Err(Error::Priority(
@@ -243,14 +292,32 @@ impl ThreadPriority {
 ///
 /// Setting thread priority to minimum with normal schedule policy:
 ///
-/// ```rust
-/// use thread_priority::*;
-///
-/// let thread_id = thread_native_id();
-/// assert!(set_thread_priority_and_policy(thread_id,
-///                                        ThreadPriority::Min,
-///                                        ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Normal)).is_ok());
-/// ```
+#[cfg_attr(
+    target_os = "macos",
+    doc = "\
+```rust
+use thread_priority::*;
+
+let thread_id = thread_native_id();
+assert!(set_thread_priority_and_policy(thread_id,
+                                       ThreadPriority::Min,
+                                       ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Other)).is_ok());
+```
+"
+)]
+#[cfg_attr(
+    not(target_os = "macos"),
+    doc = "\
+```rust
+use thread_priority::*;
+
+let thread_id = thread_native_id();
+assert!(set_thread_priority_and_policy(thread_id,
+                                       ThreadPriority::Min,
+                                       ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Normal)).is_ok());
+```
+"
+)]
 pub fn set_thread_priority_and_policy(
     native: ThreadId,
     priority: ThreadPriority,
@@ -258,6 +325,7 @@ pub fn set_thread_priority_and_policy(
 ) -> Result<(), Error> {
     let params = ScheduleParams {
         sched_priority: match policy {
+            #[cfg(not(target_os = "macos"))]
             ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => 0,
             _ => priority.to_posix(policy)?,
         },
@@ -268,6 +336,9 @@ pub fn set_thread_priority_and_policy(
 /// Set current thread's priority.
 pub fn set_current_thread_priority(priority: ThreadPriority) -> Result<(), Error> {
     let thread_id = thread_native_id();
+    #[cfg(target_os = "macos")]
+    let policy = ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Other);
+    #[cfg(not(target_os = "macos"))]
     let policy = ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Normal);
     set_thread_priority_and_policy(thread_id, priority, policy)
 }
@@ -282,7 +353,7 @@ pub fn set_current_thread_priority(priority: ThreadPriority) -> Result<(), Error
 /// assert!(thread_schedule_policy().is_ok());
 /// ```
 pub fn thread_schedule_policy() -> Result<ThreadSchedulePolicy, Error> {
-    unsafe { ThreadSchedulePolicy::from_posix(libc::sched_getscheduler(libc::getpid())) }
+    thread_schedule_policy_param(thread_native_id()).map(|policy| policy.0)
 }
 
 /// Sets thread schedule policy.
@@ -346,6 +417,8 @@ pub fn set_thread_schedule_policy(
                 &params as *const libc::sched_param,
             ),
         };
+        // This is just to silent the unused variable warning.
+        let _priority = priority;
         match ret {
             0 => Ok(()),
             e => Err(Error::OS(e)),
@@ -385,13 +458,18 @@ pub fn thread_schedule_policy_param(
     }
 }
 
-/// Get current thread's priority value.
-pub fn thread_priority() -> Result<ThreadPriority, Error> {
+/// Get the thread's priority value.
+pub fn get_thread_priority(native: ThreadId) -> Result<ThreadPriority, Error> {
     Ok(ThreadPriority::from_posix(
-        thread_schedule_policy_param(thread_native_id())?.1,
+        thread_schedule_policy_param(native)?.1,
     ))
 }
 
+/// Get current thread's priority value.
+pub fn get_current_thread_priority() -> Result<ThreadPriority, Error> {
+    get_thread_priority(thread_native_id())
+}
+
 /// A helper trait for other threads to implement to be able to call methods
 /// on threads themselves.
 ///
@@ -407,7 +485,7 @@ pub fn thread_priority() -> Result<ThreadPriority, Error> {
 /// ```
 pub trait ThreadExt {
     /// Gets the current thread's priority.
-    /// For more info read [`thread_priority`].
+    /// For more info read [`get_current_thread_priority`].
     ///
     /// ```rust
     /// use thread_priority::*;
@@ -415,7 +493,7 @@ pub trait ThreadExt {
     /// assert!(std::thread::current().get_priority().is_ok());
     /// ```
     fn get_priority(&self) -> Result<ThreadPriority, Error> {
-        thread_priority()
+        get_current_thread_priority()
     }
 
     /// Sets the current thread's priority.
@@ -451,6 +529,7 @@ pub trait ThreadExt {
     ) -> Result<(), Error> {
         let params = ScheduleParams {
             sched_priority: match policy {
+                #[cfg(not(target_os = "macos"))]
                 ThreadSchedulePolicy::Realtime(RealtimeThreadSchedulePolicy::Deadline) => 0,
                 _ => priority.to_posix(policy)?,
             },
@@ -514,23 +593,17 @@ mod tests {
     #[test]
     fn set_thread_priority_test() {
         let thread_id = thread_native_id();
+        #[cfg(not(target_os = "macos"))]
+        let policy = ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Normal);
+        #[cfg(target_os = "macos")]
+        let policy = ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Other);
 
-        assert!(set_thread_priority_and_policy(
-            thread_id,
-            ThreadPriority::Min,
-            ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Normal)
-        )
-        .is_ok());
-        assert!(set_thread_priority_and_policy(
-            thread_id,
-            ThreadPriority::Max,
-            ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Normal)
-        )
-        .is_ok());
+        assert!(set_thread_priority_and_policy(thread_id, ThreadPriority::Min, policy,).is_ok());
+        assert!(set_thread_priority_and_policy(thread_id, ThreadPriority::Max, policy,).is_ok());
         assert!(set_thread_priority_and_policy(
             thread_id,
             ThreadPriority::Crossplatform(ThreadPriorityValue(0)),
-            ThreadSchedulePolicy::Normal(NormalThreadSchedulePolicy::Normal)
+            policy,
         )
         .is_ok());
     }
